/******************************************************************************/
/*** Template class for matrix operations                                   ***/
/***------------------------------------------------------------------------***/
/*** Ingmar Bauer 						                                    ***/
/*** 31.02.2003                                                             ***/
/******************************************************************************/

/**Matrix Template-Class
  *@author Ingmar Bauer
  */
#include <iostream>
#include <vector>
#include <fstream>
#include <string>

using namespace std;

typedef vector<double> dblVect;
typedef vector<int> intVect;
typedef vector<vector <int> > intMatrix;
typedef vector<vector <double> > dblMatrix;
typedef dblVect::iterator dblVectIt;

template<class T> class matrix : vector<vector <T> > {
public:
  matrix<T>(int n, double *a);
  matrix<T>(int n);
  matrix<T>();
    
  ~matrix<T>();
  void print(void);

  int save(string fname);
  int load(string fname);

  void transpose(void);
  matrix<T> operator*(matrix<T> &a);
  
public:
  unsigned int n;
  vector<vector <T> > m;
};


template<class T> void matrix<T>::transpose(void) {
  // do nothing up to now
}

template<class T> matrix<T> matrix<T>::operator*(matrix<T> &a) {
  static matrix<T> result = matrix<T>(n);
    
  for(int y=0; y<n; y++) {
    for(int x=0; x<n; x++) {
      for(int i=0; i<n; i++) {
        result.m[y][x] += (this->m[y][i]*a.m[i][x]);
      }
    }
  }
  return result;
}

template<class T> void matrix<T>::print() {
  for(int y=0; y<n; y++) {
    for(int x=0; x<n; x++) {
      cout << m[y][x] << " ";
    }
    cout << endl;
  }
  cout << endl;
}

template<class T> matrix<T>::matrix(int n, double *a) {
  this->n = n;
  this->m = vector<vector<T> >(n, vector<T>(n));
    
  for(int i=0; i<n; i++) {
    for(int j=0; j<n; j++) {
      this->m[i][j] = a[i*n+j];
    }
  }
}

template<class T> matrix<T>::matrix(int n) {
  this->n = n;
  this->m = vector<vector<T> >(n, vector<T>(n));
    
  for(int i=0; i<n; i++) {
    for(int j=0; j<n; j++) {
      this->m[i][j] = 0.0;
    }
  }
}

template<class T> matrix<T>::matrix() {
    this->n = 0;
}

template<class T> matrix<T>::~matrix() {
}

template<class T> int matrix<T>::save(string str_fname) {

  std::ofstream f_matrix(str_fname.c_str(), ios::out | ios::binary);

  if(!f_matrix) {
    cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << endl;
    return 0;
  }

  f_matrix.write((char *)&n, 4);

  // tupel: n_el, el_1..el_n
  unsigned int n_el=0;
  unsigned int i, j;
  unsigned long n_elpos=0;
  
  for(j=0; j<this->n; j++) {

    f_matrix.write((char *)&j, 4);  // save line number
    
    n_elpos = f_matrix.tellp();
    f_matrix.write("\0\0\0\0", 4);  // placeholder for n_el, will be updated later
    n_el = 0;

    // check a line and save positions
    for(i=0; i<this->n; i++) {
      if(m[j][i] > 0) {
        n_el++;
        f_matrix.write((char *)&i, 4);
        f_matrix.write((char *)&m[j][i], sizeof(m[j][i]));
      }
    }

    // all elements added, now update n_el
    
    if(n_el > 0) {
      f_matrix.seekp(n_elpos, ios_base::beg);
      f_matrix.write((char *)&n_el, 4);
      f_matrix.seekp(0, ios_base::end);
    } else {
      f_matrix.seekp(n_elpos-4, ios_base::beg); // if empty line save nothing
    }
  }
  
  f_matrix.close();
  return 1;
}

template<class T> int matrix<T>::load(string str_fname) {
  std::ifstream f_matrix(str_fname.c_str(), ios::in | ios::binary);

  if(!f_matrix) {
    cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << endl;
    return 0;
  }

  f_matrix.read((char *)&n, 4);

  unsigned int i, j, pos;
  unsigned int n_el=0;
  unsigned int line=0;

  for(j=0; j<this->n; j++) {

    f_matrix.read((char *)&line, 4);
    f_matrix.read((char *)&n_el, 4);

    m[j].clear();
    m[j].insert(m[j].begin(), n, 0.0);
    
    for(i=0; i<n_el; i++) {
      f_matrix.read((char *)&pos, 4);
      f_matrix.read((char *)&m[line][pos], 4);
    }
  }

  f_matrix.close();
  return 1;
}
