#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <vector>

using namespace std;

#define DEBUG

#include "matrix.h"

typedef vector<float> fltVect;
typedef vector<int> intVect;
typedef vector<vector <float> > fltMatrix;
typedef fltVect::iterator fltVectIt;

#include "datasource.h"

// feste Laenge eines Indexeintrages
#define INDEX_SIZE (4+4+2+4)

int DataSource::getMatrixSize() {
    int pos_index = f_index->tellg();
    
    f_index->seekg(0, ios_base::end);
    int filesize = f_index->tellg();
    f_index->seekg(pos_index);
    
    return (filesize / INDEX_SIZE);
}

/*****************************************************************************/
/*** Constructor							   ***/
/*****************************************************************************/
DataSource::DataSource(string str_fname) {

#ifdef DEBUG
cout << "DSrc_File::DSrc_File: running." << endl;
#endif

    ready = true;
    f_index = new std::ifstream( (str_fname+string(".idx")).c_str());
    if(!*f_index) {
	cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << ".idx" << endl;
	ready=false;
    }
    
    f_refs = new std::ifstream( (str_fname+string(".ref")).c_str());
    if(!*f_refs) {
	cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << ".ref" << endl;
	ready=false;
    }

    f_url = new std::ifstream( (str_fname+string(".url")).c_str());
    if(!*f_url) {
	cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << ".url" << endl;
	ready=false;
    }
    
    offs_ref = 0;
    offs_url = 0;
    size = this->getMatrixSize();
}


/*****************************************************************************/
/*** Destructor								   ***/
/*****************************************************************************/
DataSource::~DataSource() {

#ifdef DEBUG
cout << "DSrc_File::~DSrc_File: closing index files." << endl;
#endif
    ready=false;
    f_url->close();
    f_refs->close();
    f_index->close();
}


/*****************************************************************************/
/*** int DSrc_File::getRefs()						   ***/
/***-----------------------------------------------------------------------***/
/*** Initialisiert fuer nextRef						   ***/
/*****************************************************************************/
int DataSource::getRefs() {
    f_index->seekg(0, ios_base::beg);
    f_refs->seekg(0, ios_base::beg);
    f_url->seekg(0, ios_base::beg);

    return 0;
}

/*****************************************************************************/
/*** int DSrc_File::nextRef(int &src_id, vector<int> &lnk_ids)		   ***/
/***-----------------------------------------------------------------------***/
/*** Holt die jeweils naechste src_id mit den zugehoerigen ref_ids und     ***/
/*** fuellt die uebergebenen Variablen.					   ***/
/*****************************************************************************/
int DataSource::nextRef(int &src_id, vector<int> &lnk_ids) {
    //char ch;
    
    if(ready) {
	int currpos = f_index->tellg();
	
	cout << "currpos: " << currpos << endl;
	//cout << "size   : " << size << endl;
	
	if(currpos >= size*INDEX_SIZE) {	// vectorsize * elements = filesize
	    return -1;
	}
	
	// naechste Source Id lesen
	src_id = 0x000000FF;
	src_id &= f_index->get();
	src_id |= (int)(f_index->get() << 8);
	src_id |= (int)(f_index->get() << 16);
	src_id |= (int)(f_index->get() << 24);

	// Offset fuer Refs lesen
	offs_ref = 0x000000FF;
	offs_ref &= f_index->get();
	offs_ref |= (int)(f_index->get() << 8);
	offs_ref |= (int)(f_index->get() << 16);
	offs_ref |= (int)(f_index->get() << 24);

	// Anzahl der Refs lesen
	n_refs = 0x00FF;
	n_refs = f_index->get();
	n_refs |= (int)(f_index->get() << 8);
	
	// Offset fuer Refs lesen
	offs_url = 0x000000FF;
	offs_url &= f_index->get();
	offs_url |= (int)(f_index->get() << 8);
	offs_url |= (int)(f_index->get() << 16);
	offs_url |= (int)(f_index->get() << 24);
	
	cout.setf(ios_base::hex, ios_base::basefield);
	cout << "src_id  : 0x" << src_id << endl;
	cout << "offs_ref: 0x" << offs_ref << endl;
	cout << "offs_url: 0x" << offs_url << endl;
	cout << "n_refs  : 0x" << n_refs << endl;
	
	f_refs->seekg(offs_ref);
	lnk_ids.clear();
	int help=0;
	
	for(int i=0; i<n_refs; i++) {
	    help = 0x000000FF;
	    help &= f_refs->get();
	    help |= (int)(f_refs->get() << 8);
	    help |= (int)(f_refs->get() << 16);
	    help |= (int)(f_refs->get() << 24);
	    //f_refs->get((char *)help, sizeof(int));
	    
	    lnk_ids.push_back(help);
	}
	
	//cout << "size: " << lnk_ids.size() << endl;
	//cout << "ref[0]: 0x" << lnk_ids[0] << endl;
	//cout << "ref[1]: 0x" << lnk_ids[1] << endl;
	
	return lnk_ids.size();
    }

    return -1;
}




string DataSource::getURL(int id) {

    unsigned long pos_index = f_index->tellg();
    unsigned long pos_url = f_url->tellg();
    string url;    
    
    f_index->seekg(0);	// file beginning
    
    while(!f_index->eof()) {
	int src_id = 0x000000FF;
	src_id &= f_index->get();
	src_id |= (int)(f_index->get() << 8);
	src_id |= (int)(f_index->get() << 16);
	src_id |= (int)(f_index->get() << 24);
	
	if(src_id == id) {
	    f_index->seekg(+6, ios_base::cur);
	    int offs_url = 0x000000FF;
	    offs_url &= f_index->get();
	    offs_url |= (int)(f_index->get() << 8);
	    offs_url |= (int)(f_index->get() << 16);
	    offs_url |= (int)(f_index->get() << 24);
	
	    f_url->seekg(offs_url);
	    char help[256];
	    f_url->getline(help, 255, '\n');
	    url = string(help);
	    break;
	} else if(src_id > id) {	// stop it, we're too far already
	    break;
	}
	
	f_index->seekg(+10, ios_base::cur);	// jump directly to next src_id
    }
    
    f_url->seekg(pos_url);
    f_index->seekg(pos_index);
    return url;
}

/*****************************************************************************/
/***									   ***/
/*****************************************************************************/
int DataSource::getAdjMatrix(matrix<float> &m) {

    int src_id;
    vector<int> lnk_ids;
    
    cout << "Starting filling up matrix..." << endl;
    
    this->getRefs();		// init index reader
    
    while(this->nextRef(src_id, lnk_ids) != -1) {
    
	// now update the matrix
	vector<int>::iterator v;
	for(v=lnk_ids.begin();v!=lnk_ids.end(); v++) {
	    float dummy = (float)(1.0/(float)lnk_ids.size());
	    
	    m.m[*v][src_id] = dummy;
	    //cout << "m["<<src_id<<"]["<<*v<<"] = "<<m.m[src_id][*v]<<endl;
	}
    }

    return(1);
}

/*****************************************************************************/

#ifdef TEST
int main(int argc, char *argv[]) {
    int src_id;
    vector<int> lnk_ids;
    
    DataSource dsfile("test");
    
    dsfile.nextRef(src_id, lnk_ids);
    cout << "url: " << dsfile.getURL(src_id) << endl;
    
    dsfile.nextRef(src_id, lnk_ids);
    cout << "url: " << dsfile.getURL(src_id) << endl;
    return 1;
}
#endif
